@@ -9,8 +9,8 @@
 #
 package Test::Moose::More;
 our $AUTHORITY = 'cpan:RSRCHBOY';
-# git description: 0.031-18-g6b3000b
-$Test::Moose::More::VERSION = '0.032';
+# git description: 0.034-2-g96f4433
+$Test::Moose::More::VERSION = '0.035';
 
 # ABSTRACT: More tools for testing Moose packages
 
@@ -289,7 +289,25 @@ sub check_sugar_ok($) {
 
 
 
-sub validate_thing {
+sub validate_thing { _validate_subtest_wrapper(\&_validate_thing_guts, @_) }
+sub validate_class { _validate_subtest_wrapper(\&_validate_class_guts, @_) }
+sub validate_role  { _validate_subtest_wrapper(\&_validate_role_guts,  @_) }
+
+sub _validate_subtest_wrapper {
+    my ($func, $thing, %args) = @_;
+
+    # note incrementing by 2 because of our upper curried function
+    local $Test::Builder::Level = $Test::Builder::Level + 2;
+
+    # run tests w/o a subtest wrapper...
+    return $func->($thing => %args)
+        unless $args{-subtest};
+
+    # ...or with one.
+    return $tb->subtest(delete $args{-subtest} => sub { $func->($thing => %args) });
+}
+
+sub _validate_thing_guts {
     my ($thing, %args) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -326,9 +344,10 @@ sub validate_thing {
                     if (find_meta($thing)->isa('Moose::Meta::Role'));
 
                 local $THING_NAME = "${thing}'s attribute $name";
-                $tb->subtest("checking $THING_NAME" => sub {
-                    _validate_attribute($att, %$opts);
-                });
+                _validate_attribute($att => (
+                    -subtest => "checking $THING_NAME",
+                    %$opts,
+                ));
             }
         }
     }
@@ -336,7 +355,7 @@ sub validate_thing {
     return;
 }
 
-sub validate_class {
+sub _validate_class_guts {
     my ($class, %args) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -355,15 +374,28 @@ sub validate_class {
     return validate_thing $class => %args;
 }
 
-sub validate_role {
+# _validate_role_guts() is where the main logic of validate_role() lives;
+# we're broken out here so as to allow it all to be easily wrapped -- or not
+# -- in a subtest.
+
+sub _validate_role_guts {
     my ($role, %args) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
     # basic role validation
     return unless is_role_ok $role;
+
     requires_method_ok($role => @{ $args{required_methods} })
         if defined $args{required_methods};
+
+    role_wraps_before_method_ok($role => @{ $args{before} })
+        if defined $args{before};
+    role_wraps_around_method_ok($role => @{ $args{around} })
+        if defined $args{around};
+    role_wraps_after_method_ok($role => @{ $args{after} })
+        if defined $args{after};
+
     $args{-compose}
         ?        validate_thing $role => %args
         : return validate_thing $role => %args
@@ -382,17 +414,23 @@ sub validate_role {
         : [ @{$args{required_methods}                    || []} ]
         ;
     delete $args{required_methods};
+    # and add a test for the role we're actually testing...
+    $args{does} = [ $role, @{ $args{does} || [] } ];
 
     # aaaand a subtest wrapper to make it easier to read...
-    return $tb->subtest('role composed into ' . $anon->name
-        => sub { validate_class $anon->name => %args },
+    return validate_class $anon->name => (
+        -subtest => 'role composed into ' . $anon->name,
+        %args,
     );
 }
 
 
+sub _validate_attribute { _validate_subtest_wrapper(\&__validate_attribute_guts,                 @_) }
+sub validate_attribute  { _validate_subtest_wrapper( \&_validate_attribute_guts, [shift, shift], @_) }
 
-sub validate_attribute {
-    my ($thing, $name, %opts) = @_;
+sub _validate_attribute_guts {
+    my ($thingname, %opts) = @_;
+    my ($thing, $name) = @$thingname;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unless has_attribute_ok($thing, $name);
@@ -401,7 +439,7 @@ sub validate_attribute {
     return _validate_attribute($att, %opts);
 }
 
-sub _validate_attribute {
+sub __validate_attribute_guts {
     my ($att, %opts) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -562,7 +600,7 @@ __END__
 
 =encoding UTF-8
 
-=for :stopwords Chris Weyl Chad Etheridge Granum Karen
+=for :stopwords Chris Weyl Chad Etheridge Granum Karen subtest
 
 =for :stopwords Wishlist flattr flattr'ed gittip gittip'ed
 
@@ -572,7 +610,7 @@ Test::Moose::More - More tools for testing Moose packages
 
 =head1 VERSION
 
-This document describes version 0.032 of Test::Moose::More - released July 11, 2015 as part of Test-Moose-More.
+This document describes version 0.035 of Test::Moose::More - released August 26, 2015 as part of Test-Moose-More.
 
 =head1 SYNOPSIS
 
@@ -596,7 +634,7 @@ tests, and re-exports any tests that it has and we do not, yet.
 
 Returns a list of all the known standard Moose sugar (has, extends, etc).
 
-=head1 TEST_FUNCTIONS
+=head1 TEST FUNCTIONS
 
 =head2 meta_ok $thing
 
@@ -710,6 +748,13 @@ metaclass.
 
 =item *
 
+-subtest => 'subtest name...'
+
+If set, all tests run will be wrapped in a subtest, the name of which will be
+whatever C<-subtest> is set to.
+
+=item *
+
 isa => [ ... ]
 
 A list of superclasses thing should have.
@@ -811,10 +856,41 @@ e.g.:
 
 =item *
 
+-subtest => 'subtest name...'
+
+If set, all tests run will be wrapped in a subtest, the name of which will be
+whatever C<-subtest> is set to.
+
+=item *
+
 required_methods => [ ... ]
 
 A list of methods the role requires a consuming class to supply.
 
+=item *
+
+before => [ ... ]
+
+A list of methods the role expects to wrap before, on application to a class.
+
+See L<Moose/before> for information on before method modifiers.
+
+=item *
+
+around => [ ... ]
+
+A list of methods the role expects to wrap around, on application to a class.
+
+See L<Moose/around> for information on around method modifiers.
+
+=item *
+
+after => [ ... ]
+
+A list of methods the role expects to wrap after, on application to a class.
+
+See L<Moose/after> for information on after method modifiers.
+
 =back
 
 =head2 validate_class
@@ -846,6 +922,13 @@ additional class-specific tests.
 
 =item *
 
+-subtest => 'subtest name...'
+
+If set, all tests run will be wrapped in a subtest, the name of which will be
+whatever C<-subtest> is set to.
+
+=item *
+
 immutable => 0|1
 
 Checks the class to see if it is/isn't immutable.
@@ -894,11 +977,79 @@ instance rather than a setting on the attribute; that is, '-does' ensures that t
 metaclass does a particular role (e.g. L<MooseX::AttributeShortcuts>), while 'does' tests
 the setting of the attribute to require the value do a given role.
 
+This function takes all the options L</attribute_options_ok> takes, as well as
+the following:
+
+=over 4
+
+=item *
+
+-subtest => 'subtest name...'
+
+If set, all tests run will be wrapped in a subtest, the name of which will be
+whatever C<-subtest> is set to.
+
+=back
+
 =head2 attribute_options_ok
 
 Validates that an attribute is set up as expected; like validate_attribute(),
 but only concerns itself with attribute options.
 
+Note that some of these options will skip if used against attributes defined in a role.
+
+=over 4
+
+=item *
+
+is => ro|rw
+
+Tests for reader/writer options set as one would expect.
+
+=item *
+
+isa => ...
+
+Validates that the attribute requires its value to be a given type.
+
+=item *
+
+does => ...
+
+Validates that the attribute requires its value to do a given role.
+
+=item *
+
+builder => '...'
+
+Validates that the attribute expects the method name given to be its builder.
+
+=item *
+
+default => ...
+
+Validates that the attribute has the given default.
+
+=item *
+
+init_arg => '...'
+
+Validates that the attribute has the given initial argument name.
+
+=item *
+
+lazy => 0|1
+
+Validates that the attribute is/isn't lazy.
+
+=item *
+
+required => 0|1
+
+Validates that setting the attribute's value is/isn't required.
+
+=back
+
 =for Pod::Coverage is_anon is_class is_not_anon is_role
 
 =head1 SEE ALSO
